package fr.velossity.sample.device.impl;

import java.util.Random;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.service.event.EventAdmin;

import fr.velossity.sample.device.Device;
import fr.velossity.sample.device.Measurement;

/**
 * 
 * @author C. Saint-Marcel
 * 
 */
public class MockDeviceImpl implements Device, Runnable {

	private String identifier;

	private Thread measureGenerator;

	private BundleContext myContext;

	private Measurement lastMeasurement;

	/*
	 * Default constructor.
	 */
	public MockDeviceImpl(String identifier, BundleContext context) {
		super();
		myContext = context;
		this.identifier = identifier;
	}

	public String getIdentifier() {
		return identifier;
	}

	public String getType() {
		return "Mock";
	}

	/**
	 * Mock device is always functional
	 */
	public String getState() {
		return DEVICE_STATE_FUNCTIONAL;
	}

	public Measurement getLastMeasure() {
		return lastMeasurement;
	}

	public synchronized void activate() {
		if (measureGenerator == null) {
			measureGenerator = new Thread(this);
			measureGenerator.start();
		}
	}

	public synchronized void deactivate() {
		measureGenerator = null;
	}

	public void run() {
		Thread mg = measureGenerator;
		while (measureGenerator == mg) {
			// Service class name
			String clazz = EventAdmin.class.getName();

			// REQUESTS the EventAdmin
			ServiceReference sr = myContext.getServiceReference(clazz);
			if (sr != null) {
				// BINDS to the consumer
				EventAdmin eventAdmin = (EventAdmin) myContext.getService(sr);
				lastMeasurement = new Measurement(MockDeviceImpl.this, new Random().nextFloat()* 1000, "Watt", System.currentTimeMillis());
				// POST asynchronously an event
				eventAdmin.postEvent(lastMeasurement);
			}
			try {
				synchronized (this) {
					wait(10000);
				}
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}

	}

}
